module Fulltest where
import qualified Trie as T
import TranslateHaskell  

import Test.Tasty
import Test.Tasty.HUnit
import qualified Test.Tasty.QuickCheck as QC
import qualified Test.Tasty.SmallCheck as SC

-- sample values (figure in the assignment)

dog   = "dog"
hund  = "Hund"
doS   = "do"
tun   = "tun"
car   = "car"
auto  = "Wagen"
cat   = "cat"
katze = "Katze"
cap   = "cap"
kappe = "Kappe"

l0 = []
l1 = [(dog, hund )]
l2 =  (doS, tun  ) : l1
l3 =  (car, auto ): l2
l4 =  (cat, katze) : l3
l5 =  (cap, kappe) : l4

pairs =[l0, l1, l2, l3, l4, l5]

t0, t1, t2, t3, t4, t5 :: T.Trie Char String
t0 = T.fromList l0
t1 = T.fromList l1
t2 = T.fromList l2
t3 = T.fromList l3
t4 = T.fromList l4
t5 = T.fromList l5

tries =[t0, t1, t2, t3, t4, t5]


-- test assumptions:
-- o the Map datatype is tested already
-- o Trie has no implementation invariants that have to be checked
-- o thus, only the laws for maps have to be tested for Tries
-- o the key property of germanHaskell is checked manually

run = defaultMain $ testGroup "All tests"
  [insertTests1, insertTests2, deleteTests1, deleteTests2, toListTests1,
   lookupTests1, lookupTests2]

-- x == y ==> insert (x,fx) $ insert (y, fy) t == insert (x,fx) t

insertTests1 = testGroup "insert-insert-idempotence"
    [ testCase "t0" $
        T.insert "comb" "Kamm" (T.insert "comb" "Hahnenkamm" t0)
          @=? T.insert "comb" "Kamm" t0
    , testCase "t5" $
        T.insert "comb" "Kamm" (T.insert "comb" "Hahnenkamm" t5)
          @=? T.insert "comb" "Kamm" t5
    ]

-- x /= y ==> insert (x,fx) $ insert (y, fy) t
--            == insert (y, fy) (insert (x,fx) t)


insertTests2 = testGroup "insert-insert-commutation"
    [ testCase "t0" $
        T.insert "comb" "Kamm" (T.insert  "day" "Tag" t0)
          @=? T.insert  "day" "Tag" (T.insert  "comb" "Kamm" t0)
    , testCase "t5" $
        T.insert  "comb" "Kamm" (T.insert "day" "Tag" t5)
          @=? T.insert "day" "Tag" (T.insert "comb" "Kamm" t5)
    ]

-- x == y ==> lookup x (insert (y, fy) t) == Just fy

-- x /= y ==> lookup x (insert (y, fy) t) == lookup x t

-- x == y ==> delete x (insert (y, fy) t) == t

deleteTests1 = testGroup "delete-insert-extinction"
    [ testCase "t0" $
        T.delete "comb" (T.insert "comb" "Hahnenkamm" t0)
          @=? T.delete "comb" t0
    , testCase "t5" $
        T.delete "comb" (T.insert "comb" "Hahnenkamm" t5)
          @=? T.delete "comb" t5
    ]

-- x /= y ==> delete x (insert (y, fy) t) == insert (y, fy) (delete x t)

-- x `notElem` keys t ==> delete x t == t

deleteTests2 = testGroup "delete-neutrality"
    [ testCase "t0" $ T.delete "comb" t0 @=? t0
    , testCase "t5" $ T.delete "comb" t5  @=? t5
    ]


-- T.fromList . T.toList $ x  == x

-- toList . fromList $ x  == x
-- This does *not* hold in general
-- - Argument of fromList needs not be sorted
-- - Argument need not be right-unique

toListTests1 = testGroup "toList-fromList-duality"
    [ testCase "t0" $ T.fromList (T.toList t0) @=? t0
    , testCase "t5" $ T.fromList (T.toList t5) @=? t5
    ]


lookupTests1 = testGroup "lookup"
    [ testCase "lookup car" $ T.lookup car t5 @?= Just auto
    , testCase "lookup do" $ T.lookup doS t5 @?= Just tun
    , testCase "lookup foo" $ T.lookup "foo" t5 @?= Nothing
    , testCase "lookup \"\"" $ T.lookup "" t5 @?= Nothing
    ]

t6 = T.insert car "Auto" t5
t7 = T.delete car t6


lookupTests2 = testGroup "lookup/insert/delete"
    [ testCase "lookup/insert same" $ T.lookup car t6 @?= Just "Auto"
    , testCase "lookup/delete same" $ T.lookup car t7 @?= Nothing
    , testCase "lookup/insert diff" $ T.lookup "cast" t6 @?= Nothing
    , testCase "lookup/delete diff" $ T.lookup cat t7 @?= Just katze
    ]

